"""An image engine implements image processing operators and an abstract image format.

Using these abstract classes allows the user to write image processing code that can run with various different image engines (eg imagej or opencv).
"""

import abc

ABC = abc.ABCMeta('ABC', (object,), {})


class IImage(ABC):

    @abc.abstractmethod
    def width(self):
        pass

    @abc.abstractmethod
    def height(self):
        pass


class IImageEngine(ABC):
    """
    The active image processor.

    As there should only be one image engine instance at any given time, this class is implemented as a singleton
    """

    # Here will be the instance stored.
    __instance = None

    @staticmethod
    def set_instance(image_engine):
        """Change the active image_engine.

        :param IImageEngine image_engine:
        """
        IImageEngine.__instance = image_engine

    @staticmethod
    def get_instance():
        """Static access method."""
        assert IImageEngine.__instance is not None
        return IImageEngine.__instance

    @abc.abstractmethod
    def save_as_tiff(self, image, out_file_path):
        """
        :param IImage image:
        :param str out_file_path: eg './white_estimate.tiff'
        """


    @abc.abstractmethod
    def compute_max(self, images_file_path):
        """Compute for each pixel position the maximum at this position in all input images.

        :param list(str) images_file_path:
        :rtype IImage:
        """

